/**
 * File:
 *  main.c
 *
 * Notes:
 *  This application demonstrates the modbus master implementation
 *  Uses UART1 routines for communicating with serial RS232 port
 *
 * History:
 *  08/07/2012 Written by Simon Platten
 */
#include "modbusMaster.h"

static volatile modbusBlockDef coilsBlock,
                               statusBitsBlock, 
                               inputRegsBlock, 
                               holdingRegsBlock;
static volatile uint aryuintInputRegs[1], aryuintHoldingRegs[2];
static volatile byte arybytCoils[1], arybytStatusBits[1];
static volatile modbusPollScheduleDef coilsSchedule,
                                      statusSchedule,
                                      inputRegSchedule,
                                      holdingRegSchedule;
/**
 * Interrupt Service Routine
 */
void interrupt() {
  decodePacket();
}
/**
 * Callback routine called when coils are requested
 */
void coilsCallBack(void) {
  LATB = ((byte*)pCurrBlock->paryData)[0];
}
/**
 * Callback routine called when status inputs are requested
 */
void statusCallBack(void) {
///  LATE = ((byte*)pCurrBlock->paryData)[0];
}
/**
 * Callback routine called when holding registers are requested
 */
void holdingRegCallBack(void) {
  LATB = ((uint*)pCurrBlock->paryData)[0];
}
/**
 * Entry point
 */
void main() {
  byte bytData[16];
  
  ANSELB = 0;
  ANSELD = 0;
// Configure PORTB as output
  TRISB = 0;
// PORTD as input
  TRISD = 0xff;
// Initialise ports
  LATB = 0;
// Create an instance of the modbus slave
  modbusSerialInit(BAUD_115200, 1);
// Make sure the data areas are all cleared
  memset(arybytCoils,        0, sizeof(arybytCoils));
  memset(arybytStatusBits,   0, sizeof(arybytStatusBits));
  memset(aryuintHoldingRegs, 0, sizeof(aryuintHoldingRegs));
  memset(aryuintInputRegs,   0, sizeof(aryuintInputRegs));
// Create the various data I/O blocks
  addModbusBlock(1, COILS,             &coilsBlock,       1, 8,
                 (void*)arybytCoils, coilsCallBack);
  addModbusBlock(1, STATUS_INPUTS,     &statusBitsBlock,  1, 8,
                 (void*)arybytStatusBits, statusCallBack);
  addModbusBlock(1, HOLDING_REGISTERS, &holdingRegsBlock, 1, 2,
                 (void*)aryuintHoldingRegs, holdingRegCallBack);
  addModbusBlock(1, INPUT_REGISTERS,   &inputRegsBlock,   1, 1,
                 (void*)aryuintInputRegs, NULL);
// Define polling schedule
  scheduleRequest(&coilsBlock,       &coilsSchedule);
  scheduleRequest(&statusBitsBlock,  &statusSchedule);
  scheduleRequest(&holdingRegsBlock, &holdingRegSchedule);
  scheduleRequest(&inputRegsBlock,   &inputRegSchedule);
// Endless loop
  while (1) {
    serviceSchedule();
    
    if ( Button(&PORTD, 0, 1, 1) ) {
       bytData[0] = 0x01;
       modifyBlock(&coilsSchedule, 1, 1, bytData);
    } 
    if ( Button(&PORTD, 1, 1, 1) ) {
       bytData[0] = 0x3d;
       modifyBlock(&coilsSchedule, 1, 7, bytData);
    } 
    if ( Button(&PORTD, 2, 1, 1) ) {
       bytData[0] = 0xca;
       bytData[1] = 0xca;
       modifyBlock(&holdingRegSchedule, 1, 1, bytData);
    } 
    if ( Button(&PORTD, 3, 1, 1) ) {
       bytData[0] = 0x04;
       bytData[1] = 0xd2;
       bytData[2] = 0x16;
       bytData[3] = 0x2e;
       modifyBlock(&holdingRegSchedule, 1, 2, bytData);
    }
  }
}