/**
 * File:
 *  modbusMaster.h
 *
 * Notes:
 *  This file contains the prototype for a modbus RTU master implementation.
 *
 * History:
 *  14/07/2012 Written by Simon Platten
 *  22/07/2012 Added modifyBlock function
 *  31/08/2012 Added:
 *             pModify            - Function pointer
 *             uintItemsToModify  - Total items to modify
 *             uintStartAddress   - Start address to modify
 *             arybytPendingWrite - Write data buffer
 */
#ifndef MODBUS_MASTER_H
  #define MODBUS_MASTER_H
  
  #include "modbus.h"

// Inter-poll delay, in milliseconds
  #define MODBUS_INTERPOLL_DELAY     0//5000
// Block status
  typedef enum eModbusBlockStatus {
    BLOCK_STATUS_UNKNOWN              = 0,
    BLOCK_STATUS_READY                = 1,
    BLOCK_STATUS_SUSPEND              = 2,
    BLOCK_STATUS_WRITE                = 3,
    BLOCK_STATUS_WAITING_FOR_RESPONSE = 4,
    BLOCK_STATUS_RETRY                = 5
  } mbBlockStatus;
// Polling schedule definition
  typedef struct _modbusPollSchedule {
// Pointer to the I/O block
    modbusBlockDef* pBlock;
// Block status, see eModbusBlockStatus
    mbBlockStatus eStatus;
// The last Function Code issued to slave
    byte bytLastFC;
// Poll packet length
    uint uintPL;
// Expected response length
    uint uintERL;
// Function pointer to modify block
    void (*pModify)(struct _modbusPollSchedule* pSchedule);
// The number of items to modify
    uint uintItemsToModify;
// The start address for writing
    uint uintStartAddress;
// The data area for any pending write
    byte arybytPendingWrite[MAX_REGISTERS_IN_FC16 * 2];
// Link to next block
    struct _modbusPollSchedule* pNext;
  } modbusPollScheduleDef;
// Prototypes
  void    decodePacket(void);
  boolean modifyBlock(modbusPollScheduleDef* pSchedule, 
                          uint uintAddress, uint uintTotal, byte* pData);
  boolean scheduleRequest(modbusBlockDef* pBlock,
                          modbusPollScheduleDef* pSchedule);
  void    serviceSchedule(void);
  void    unpackBits(byte* pBuffer);
  void    unpackWords(byte* pBuffer);
#endif